@@ -16,19 +16,19 @@
 # under the License.
 
 import logging
-import os
 import platform
-import socket
 import string
+import warnings
 from base64 import b64encode
+from typing import Optional
 from urllib import parse
 
-import certifi
 import urllib3
 
 from selenium import __version__
 
 from . import utils
+from .client_config import ClientConfig
 from .command import Command
 from .errorhandler import ErrorCode
 
@@ -136,12 +136,27 @@ class RemoteConnection:
     """
 
     browser_name = None
+    # Keep backward compatibility for AppiumConnection - https://github.com/SeleniumHQ/selenium/issues/14694
+    import os
+    import socket
+
+    import certifi
+
     _timeout = (
         float(os.getenv("GLOBAL_DEFAULT_TIMEOUT", str(socket.getdefaulttimeout())))
         if os.getenv("GLOBAL_DEFAULT_TIMEOUT") is not None
         else socket.getdefaulttimeout()
     )
     _ca_certs = os.getenv("REQUESTS_CA_BUNDLE") if "REQUESTS_CA_BUNDLE" in os.environ else certifi.where()
+    _client_config: ClientConfig = None
+
+    system = platform.system().lower()
+    if system == "darwin":
+        system = "mac"
+
+    # Class variables for headers
+    extra_headers = None
+    user_agent = f"selenium/{__version__} (python {system})"
 
     @classmethod
     def get_timeout(cls):
@@ -150,7 +165,12 @@ class RemoteConnection:
         Timeout value in seconds for all http requests made to the
         Remote Connection
         """
-        return None if cls._timeout == socket._GLOBAL_DEFAULT_TIMEOUT else cls._timeout
+        warnings.warn(
+            "get_timeout() in RemoteConnection is deprecated, get timeout from ClientConfig instance instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return cls._client_config.timeout
 
     @classmethod
     def set_timeout(cls, timeout):
@@ -159,12 +179,22 @@ class RemoteConnection:
         :Args:
             - timeout - timeout value for http requests in seconds
         """
-        cls._timeout = timeout
+        warnings.warn(
+            "set_timeout() in RemoteConnection is deprecated, set timeout to ClientConfig instance in constructor instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        cls._client_config.timeout = timeout
 
     @classmethod
     def reset_timeout(cls):
         """Reset the http request timeout to socket._GLOBAL_DEFAULT_TIMEOUT."""
-        cls._timeout = socket._GLOBAL_DEFAULT_TIMEOUT
+        warnings.warn(
+            "reset_timeout() in RemoteConnection is deprecated, use reset_timeout() in ClientConfig instance instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        cls._client_config.reset_timeout()
 
     @classmethod
     def get_certificate_bundle_path(cls):
@@ -174,7 +204,12 @@ class RemoteConnection:
         command executor. Defaults to certifi.where() or
         REQUESTS_CA_BUNDLE env variable if set.
         """
-        return cls._ca_certs
+        warnings.warn(
+            "get_certificate_bundle_path() in RemoteConnection is deprecated, get ca_certs from ClientConfig instance instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return cls._client_config.ca_certs
 
     @classmethod
     def set_certificate_bundle_path(cls, path):
@@ -185,7 +220,12 @@ class RemoteConnection:
         :Args:
             - path - path of a .pem encoded certificate chain.
         """
-        cls._ca_certs = path
+        warnings.warn(
+            "set_certificate_bundle_path() in RemoteConnection is deprecated, set ca_certs to ClientConfig instance in constructor instead",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        cls._client_config.ca_certs = path
 
     @classmethod
     def get_remote_connection_headers(cls, parsed_url, keep_alive=False):
@@ -196,14 +236,10 @@ class RemoteConnection:
          - keep_alive (Boolean) - Is this a keep-alive connection (default: False)
         """
 
-        system = platform.system().lower()
-        if system == "darwin":
-            system = "mac"
-
         headers = {
             "Accept": "application/json",
             "Content-Type": "application/json;charset=UTF-8",
-            "User-Agent": f"selenium/{__version__} (python {system})",
+            "User-Agent": cls.user_agent,
         }
 
         if parsed_url.username:
@@ -213,13 +249,10 @@ class RemoteConnection:
         if keep_alive:
             headers.update({"Connection": "keep-alive"})
 
-        return headers
+        if cls.extra_headers:
+            headers.update(cls.extra_headers)
 
-    def _get_proxy_url(self):
-        if self._url.startswith("https://"):
-            return os.environ.get("https_proxy", os.environ.get("HTTPS_PROXY"))
-        if self._url.startswith("http://"):
-            return os.environ.get("http_proxy", os.environ.get("HTTP_PROXY"))
+        return headers
 
     def _identify_http_proxy_auth(self):
         url = self._proxy_url
@@ -235,10 +268,17 @@ class RemoteConnection:
         return proxy_without_auth, auth
 
     def _get_connection_manager(self):
-        pool_manager_init_args = {"timeout": self.get_timeout()}
-        if self._ca_certs:
+        pool_manager_init_args = {"timeout": self._client_config.timeout}
+        pool_manager_init_args.update(
+            self._client_config.init_args_for_pool_manager.get("init_args_for_pool_manager", {})
+        )
+
+        if self._client_config.ignore_certificates:
+            pool_manager_init_args["cert_reqs"] = "CERT_NONE"
+            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
+        elif self._client_config.ca_certs:
             pool_manager_init_args["cert_reqs"] = "CERT_REQUIRED"
-            pool_manager_init_args["ca_certs"] = self._ca_certs
+            pool_manager_init_args["ca_certs"] = self._client_config.ca_certs
 
         if self._proxy_url:
             if self._proxy_url.lower().startswith("sock"):
@@ -252,34 +292,79 @@ class RemoteConnection:
 
         return urllib3.PoolManager(**pool_manager_init_args)
 
-    def __init__(self, remote_server_addr: str, keep_alive: bool = False, ignore_proxy: bool = False):
-        self.keep_alive = keep_alive
-        self._url = remote_server_addr
+    def __init__(
+        self,
+        remote_server_addr: Optional[str] = None,
+        keep_alive: Optional[bool] = True,
+        ignore_proxy: Optional[bool] = False,
+        ignore_certificates: Optional[bool] = False,
+        init_args_for_pool_manager: Optional[dict] = None,
+        client_config: Optional[ClientConfig] = None,
+    ):
+        self._client_config = client_config or ClientConfig(
+            remote_server_addr=remote_server_addr,
+            keep_alive=keep_alive,
+            ignore_certificates=ignore_certificates,
+            init_args_for_pool_manager=init_args_for_pool_manager,
+        )
 
-        # Env var NO_PROXY will override this part of the code
-        _no_proxy = os.environ.get("no_proxy", os.environ.get("NO_PROXY"))
-        if _no_proxy:
-            for npu in _no_proxy.split(","):
-                npu = npu.strip()
-                if npu == "*":
-                    ignore_proxy = True
-                    break
-                n_url = parse.urlparse(npu)
-                remote_add = parse.urlparse(self._url)
-                if n_url.netloc:
-                    if remote_add.netloc == n_url.netloc:
-                        ignore_proxy = True
-                        break
-                else:
-                    if n_url.path in remote_add.netloc:
-                        ignore_proxy = True
-                        break
+        # Keep backward compatibility for AppiumConnection - https://github.com/SeleniumHQ/selenium/issues/14694
+        RemoteConnection._timeout = self._client_config.timeout
+        RemoteConnection._ca_certs = self._client_config.ca_certs
+        RemoteConnection._client_config = self._client_config
 
-        self._proxy_url = self._get_proxy_url() if not ignore_proxy else None
-        if keep_alive:
+        if remote_server_addr:
+            warnings.warn(
+                "setting remote_server_addr in RemoteConnection() is deprecated, set in ClientConfig instance instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if not keep_alive:
+            warnings.warn(
+                "setting keep_alive in RemoteConnection() is deprecated, set in ClientConfig instance instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if ignore_certificates:
+            warnings.warn(
+                "setting ignore_certificates in RemoteConnection() is deprecated, set in ClientConfig instance instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if init_args_for_pool_manager:
+            warnings.warn(
+                "setting init_args_for_pool_manager in RemoteConnection() is deprecated, set in ClientConfig instance instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if ignore_proxy:
+            warnings.warn(
+                "setting ignore_proxy in RemoteConnection() is deprecated, set in ClientConfig instance instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            self._proxy_url = None
+        else:
+            self._proxy_url = self._client_config.get_proxy_url()
+
+        if self._client_config.keep_alive:
             self._conn = self._get_connection_manager()
         self._commands = remote_commands
 
+    extra_commands = {}
+
+    def add_command(self, name, method, url):
+        """Register a new command."""
+        self._commands[name] = (method, url)
+
+    def get_command(self, name: str):
+        """Retrieve a command if it exists."""
+        return self._commands.get(name)
+
     def execute(self, command, params):
         """Send a command to the remote server.
 
@@ -291,7 +376,7 @@ class RemoteConnection:
          - params - A dictionary of named parameters to send with the command as
            its JSON payload.
         """
-        command_info = self._commands[command]
+        command_info = self._commands.get(command) or self.extra_commands.get(command)
         assert command_info is not None, f"Unrecognised command {command}"
         path_string = command_info[1]
         path = string.Template(path_string).substitute(params)
@@ -300,7 +385,7 @@ class RemoteConnection:
             for word in substitute_params:
                 del params[word]
         data = utils.dump_json(params)
-        url = f"{self._url}{path}"
+        url = f"{self._client_config.remote_server_addr}{path}"
         trimmed = self._trim_large_entries(params)
         LOGGER.debug("%s %s %s", command_info[0], url, str(trimmed))
         return self._request(command_info[0], url, body=data)
@@ -317,18 +402,22 @@ class RemoteConnection:
           A dictionary with the server's parsed JSON response.
         """
         parsed_url = parse.urlparse(url)
-        headers = self.get_remote_connection_headers(parsed_url, self.keep_alive)
-        response = None
+        headers = self.get_remote_connection_headers(parsed_url, self._client_config.keep_alive)
+        auth_header = self._client_config.get_auth_header()
+
+        if auth_header:
+            headers.update(auth_header)
+
         if body and method not in ("POST", "PUT"):
             body = None
 
-        if self.keep_alive:
-            response = self._conn.request(method, url, body=body, headers=headers)
+        if self._client_config.keep_alive:
+            response = self._conn.request(method, url, body=body, headers=headers, timeout=self._client_config.timeout)
             statuscode = response.status
         else:
             conn = self._get_connection_manager()
             with conn as http:
-                response = http.request(method, url, body=body, headers=headers)
+                response = http.request(method, url, body=body, headers=headers, timeout=self._client_config.timeout)
             statuscode = response.status
         data = response.data.decode("UTF-8")
         LOGGER.debug("Remote response: status=%s | data=%s | headers=%s", response.status, data, response.headers)
@@ -336,7 +425,9 @@ class RemoteConnection:
             if 300 <= statuscode < 304:
                 return self._request("GET", response.headers.get("location", None))
             if 399 < statuscode <= 500:
-                return {"status": statuscode, "value": data}
+                if statuscode == 401:
+                    return {"status": statuscode, "value": "Authorization Required"}
+                return {"status": statuscode, "value": str(statuscode) if not data else data.strip()}
             content_type = []
             if response.headers.get("Content-Type", None):
                 content_type = response.headers.get("Content-Type", None).split(";")
