@@ -15,26 +15,21 @@
 # specific language governing permissions and limitations
 # under the License.
 
-import http.client as http_client
-import warnings
-
 from selenium.common.exceptions import WebDriverException
-from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
+from selenium.webdriver.remote.client_config import ClientConfig
 from selenium.webdriver.remote.webdriver import WebDriver as RemoteWebDriver
 
+from ..common.driver_finder import DriverFinder
 from .options import Options
 from .remote_connection import SafariRemoteConnection
 from .service import Service
 
-DEFAULT_SAFARI_CAPS = DesiredCapabilities.SAFARI.copy()
-
 
 class WebDriver(RemoteWebDriver):
     """Controls the SafariDriver and allows you to drive the browser."""
 
     def __init__(
         self,
-        reuse_service=False,
         keep_alive=True,
         options: Options = None,
         service: Service = None,
@@ -43,46 +38,42 @@ class WebDriver(RemoteWebDriver):
         safaridriver service.
 
         :Args:
-         - reuse_service - If True, do not spawn a safaridriver instance; instead, connect to an already-running service that was launched externally.
          - keep_alive - Whether to configure SafariRemoteConnection to use
              HTTP keep-alive. Defaults to True.
          - options - Instance of ``options.Options``.
          - service - Service object for handling the browser driver if you need to pass extra details
         """
-        if reuse_service:
-            warnings.warn(
-                "reuse_service has been deprecated, please use the Service class to set it",
-                DeprecationWarning,
-                stacklevel=2,
-            )
-
         self.service = service if service else Service()
-        self.options = options if options else Options()
-        self.keep_alive = keep_alive
+        options = options if options else Options()
 
-        self._reuse_service = reuse_service and self.service.reuse_service
-        if not self._reuse_service:
+        self.service.path = self.service.env_path() or DriverFinder(self.service, options).get_driver_path()
+
+        if not self.service.reuse_service:
             self.service.start()
 
+        client_config = ClientConfig(remote_server_addr=self.service.service_url, keep_alive=keep_alive, timeout=120)
         executor = SafariRemoteConnection(
-            remote_server_addr=self.service.service_url,
-            keep_alive=self.keep_alive,
-            ignore_proxy=self.options._ignore_local_proxy,
+            ignore_proxy=options._ignore_local_proxy,
+            client_config=client_config,
         )
 
-        super().__init__(command_executor=executor, options=self.options)
+        try:
+            super().__init__(command_executor=executor, options=options)
+        except Exception:
+            self.quit()
+            raise
 
         self._is_remote = False
 
     def quit(self):
-        """Closes the browser and shuts down the SafariDriver executable that
-        is started when starting the SafariDriver."""
+        """Closes the browser and shuts down the SafariDriver executable."""
         try:
             super().quit()
-        except http_client.BadStatusLine:
+        except Exception:
+            # We don't care about the message because something probably has gone wrong
             pass
         finally:
-            if not self._reuse_service:
+            if not self.service.reuse_service:
                 self.service.stop()
 
     # safaridriver extension commands. The canonical command support matrix is here:
